@@ -6,6 +6,7 @@ use warnings;
 use Carp;
 use IO::File;
 use Bit::Vector;
+use Data::Dumper;
 
 =head1 NAME
 
@@ -75,13 +76,27 @@ file using Image::Embroidery.
     $gray = $im->colorAllocate(128,128,128);
     $red = $im->colorAllocate(255,0,0);
     
+    # you can control the thickness of the lines that are used to draw the 
+    # image. the default thickness is 1, which will let you see individual
+    # stitches. The higher you set the thickness, the smoother the image will
+    # look. A thickness of 3 or 4 is good for showing what the finished product
+    # will look like
+    $im->setThickness(3);
+
     # the order you specify the colors is the order in which they
     # will be used. you must specify the correct number of colors
     $emb->draw_logo($im, $gray, $red);
 
     open(IMG, ">", "/path/to/embroidery.png");
+    #  make sure you use binary mode when running on Windows
+    binmode(IMG);
     print IMG $im->png;
-    close(F);
+    close(IMG);
+
+Converting from one format to another
+
+    $emb->read_file( '/path/to/embroidery.exp', 'exp' );
+    $emb->save_file( '/path/to/embroidery.dst', 'dst' );
 
 =head1 METHODS
 
@@ -106,7 +121,7 @@ our %EXPORT_TAGS = ( 'all' => [ qw($NORMAL $JUMP $COLOR_CHANGE) ] );
 
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 
-$VERSION = '1.1';
+$VERSION = '1.2';
 
 $NORMAL = 0;
 $JUMP = 1;
@@ -121,12 +136,11 @@ The constructor.
 sub new {
 	my $proto = shift;
 	my $class = ref($proto) || $proto;
-	my $self  = {};
+	my $self  = {
+		ignore_header_coordinates => 0,
+	};
 
-	$self->{'x_size'} = 0;
-	$self->{'y_size'} = 0;
-	$self->{'filename'} = '';
-	$self->{'data'} = {};
+	$self->{'filename'} = undef;
 
 	bless ($self, $class);
 	return $self;
@@ -138,8 +152,8 @@ sub new {
   $emb->read_file($filename, 'tajima');
 
 Read an embroidery data file in the specified file format.
-If the format is omitted, the default is 'tajima'. Returns
-0 on failure, 1 on success.
+See FILE FORMATS for supported formats. Default is Tajima DST.
+Returns 0 on failure, 1 on success.
 
 =cut
 sub read_file {
@@ -151,15 +165,117 @@ sub read_file {
   
 	$self->{'filename'} = $file;
   
-	unless(defined($type)) { $type = 'tajima'; }
+	$type = (defined($type)) ? lc($type) : 'tajima';
 
-	if($type eq 'tajima') {
+	if($type eq 'tajima' or $type eq 'dst') {
 		return _read_tajima_file($self, $fh);
+	} elsif($type eq 'melco' or $type eq 'exp') {
+		return _read_melco_file($self, $fh);
 	} else {
 		carp("Request to read unknown file type!");
 	}
 }
 
+sub _read_melco_file {
+	my ($self, $fh) = @_;
+	my $record;
+	$self->{'data'} = {};
+
+	my $colorchange = '8001';
+
+	# i don't know why both of these can be used
+	# for a jump record.
+	my $jump1 = '8002';
+	my $jump2 = '8004';
+
+	# initialize pattern info, and set defaults for stuff that melco doesn't use (MX/MY/PD multi-volume data)
+	foreach my $field ('color_changes', 'stitches', '+X', '-X', '+Y', '-Y', 'MX', 'MY', 'PD') {
+		$self->{'data'}{$field} = 0;
+	}
+	$self->{'data'}{'label'} = 'FromMelco';
+
+	# current offset from the starting point
+	my $currentX = 0;
+	my $currentY = 0;
+
+	while($fh->read($record, 2)) {
+		$record = unpack('H4', $record);
+
+		my ($x, $y);
+
+		# remove empty records that are sometimes inserted after color
+		# changes.
+		if($record eq '0000') {
+			next;
+		} elsif($record eq $colorchange) {
+			push(@{$self->{'data'}{'pattern'}}, [ $COLOR_CHANGE ]);
+			$self->{'data'}{'color_changes'}++;
+			next;
+		} elsif($record eq $jump1 or $record eq $jump2) {
+			$fh->read($record, 2);
+			($x, $y) = _decode_melco_delta( unpack('H4', $record) );
+			push(@{$self->{'data'}{'pattern'}}, [ $JUMP, $x, $y ]);
+		# some generators insert 8080 records, but I don't know what they mean
+		} elsif($record =~ /^80/) {
+			$fh->read(undef, 2);
+			next;
+		} else {
+			($x, $y) = _decode_melco_delta($record);
+
+			push(@{$self->{'data'}{'pattern'}}, [ $NORMAL, $x, $y ]);
+			$self->{'data'}{'stitches'}++;
+		}
+
+		# keep track of how big the pattern is
+		$currentX += $x;
+		$currentY += $y;
+		if($currentX > $self->{'data'}{'+X'}) { $self->{'data'}{'+X'} = $currentX; }
+		if($currentX < $self->{'data'}{'-X'}) { $self->{'data'}{'-X'} = $currentX; }
+		if($currentY > $self->{'data'}{'+Y'}) { $self->{'data'}{'+Y'} = $currentY; }
+		if($currentY < $self->{'data'}{'-Y'}) { $self->{'data'}{'-Y'} = $currentY; }
+	}
+
+	# these are magnitudes, so remove the minus sign
+	$self->{'data'}{'-X'} = abs($self->{'data'}{'-X'});
+	$self->{'data'}{'-Y'} = abs($self->{'data'}{'-Y'});
+
+	# store the total size of the pattern
+	$self->{'data'}{'x_size'} = $self->{'data'}{'+X'} + $self->{'data'}{'-X'};
+	$self->{'data'}{'y_size'} = $self->{'data'}{'+Y'} + $self->{'data'}{'-Y'};
+
+	# last position
+	$self->{'data'}{'AX'} = $currentX;
+	$self->{'data'}{'AY'} = $currentY;
+
+	return 1;
+}
+
+sub _encode_melco_delta {
+	my ($x, $y) = @_;
+	if($x < 0) { $x += 256; }
+	if($y < 0) { $y += 256; }
+
+	my $delta_record = sprintf('%02x%02x', $x, $y);
+	return $delta_record;
+}
+
+sub _decode_melco_delta {
+	my ($record) = @_;
+	my $x = hex(substr($record, 0, 2));
+	my $y = hex(substr($record, 2, 2));
+	
+	# 127 is the max stitch length, 128 is a special value
+	# for encoding jumps and color changes
+	if($x == 128 or $y == 128) {
+		return (0, 0);
+	}
+
+	if($x > 127) { $x = $x - 256; }
+	if($y > 127) { $y = $y - 256; }
+
+	return ($x, $y);
+}
+
 # parse a Tajima DST file
 sub _read_tajima_file {
 	my ($self, $fh) = @_;
@@ -177,6 +293,10 @@ sub _read_tajima_file {
 			0,  0,  0,  0, -9,  9, -1,  1
 	);
 
+	# keep track of the actual color changes we see, to verify that
+	# it matches what's in the header. some programs incorrectly put
+	# the number of colors in the header, which will be one too large
+	my $actual_color_changes = 0;
 
 	# i don't think the order of these header elements
 	# can change, but i'll be flexible.
@@ -192,13 +312,14 @@ sub _read_tajima_file {
 
 		if($field eq 'LA') {
 			$fh->read(my $label, 16);
-			($self->{'data'}{'LA'} = $label) =~ s/\s*$//;
+			($self->{'data'}{'label'} = $label) =~ s/\s*$//;
 		} elsif($field eq 'ST') {
-			$fh->read($self->{'data'}{'ST'}, 7);
-			$self->{'data'}{'ST'} = int($self->{'data'}{'ST'});
+			$fh->read($self->{'data'}{'stitches'}, 7);
+			$self->{'data'}{'stitches'} = int($self->{'data'}{'stitches'});
 		} elsif($field eq 'CO') {
-			$fh->read($self->{'data'}{'CO'}, 3);
-			$self->{'data'}{'CO'} = int($self->{'data'}{'CO'});
+			my $color_changes;
+			$fh->read($color_changes, 3);
+			$self->{'data'}{'color_changes'} = int($color_changes);
 		} elsif($field =~ /^([-+][XY])$/) {
 			$fh->read(my $val, 5);
 			$self->{'data'}{"$1"} = int($val);
@@ -232,16 +353,19 @@ sub _read_tajima_file {
 
 	# the file spec for Tajima DST indicates that bits 0 and 1 of a
 	# stitch should always be '1', but since they don't mean anything,
+	# and some file generators don't follow the spec very carefully,
 	# we just require them to be consistent throughout the file.
 	# we store the values in the first stitch that we find, then 
 	# compare subsequent stitches to the first value we saw. 
 	my $stitch_bit_0;
 	my $stitch_bit_1;
 
+
 	while($fh->read($stitch, 3)) {
 		my $v = Bit::Vector->new(24);
 		$v->from_Hex(unpack('H6', $stitch));
 
+		# just check for consistency to detect corrupt files, these bits are meaningless
 		if(defined($stitch_bit_0)) {
 			unless($v->bit_test(1) == $stitch_bit_1 and $v->bit_test(0) == $stitch_bit_0) {
 				carp("Possibly corrupt data file: ", unpack('H6', $stitch));
@@ -253,20 +377,23 @@ sub _read_tajima_file {
 
 		# bit 6 is off for jumps and normal stitches
 		if(!$v->bit_test(6)) {
-			my ($x, $y);
-			foreach my $index(0..23) {
+			my ($x, $y) = (0, 0);
+			# first two bits are not used. 6 and 7 are record type flags
+			foreach my $index(2..5, 8..23) {
 				$x += $x_incr[$index] if($v->bit_test($index));
 				$y += $y_incr[$index] if($v->bit_test($index));
 			}
 
 			# bit 7 will be off for normal stitches, on for jumps
 			push(@{$self->{'data'}{'pattern'}}, [ $v->bit_test(7), $x, $y ]);
+
 		} elsif(!$v->bit_test(7)) {
 			carp("Invalid operation code");
 			return 0;
 		} else {
 			if($v->to_Hex() eq '0000C3') {
 				push(@{$self->{'data'}{'pattern'}}, [ $COLOR_CHANGE ]);
+				$actual_color_changes++;
 			} elsif($v->to_Hex() eq '0000F3') {
 				# this is the 'stop' code. sometimes there is trailing data, so
 				# stop reading now.
@@ -277,6 +404,13 @@ sub _read_tajima_file {
 			}
 		}
 	}
+
+	# trust the data more than the header
+	if($actual_color_changes != $self->{'data'}{'color_changes'}) {
+		# TODO some kind of logging ("Tajima file header lists incorrect number of color changes: $self->{'data'}{'color_changes'}, should be $actual_color_changes");
+		$self->{'data'}{'color_changes'} = $actual_color_changes;
+	}
+
 	return 1;
 }
 
@@ -284,14 +418,14 @@ sub _read_tajima_file {
 
   $emb->write_file();
   $emb->write_file( $filename );
-  $emb->write_file( $filename, 'tajima' );
+  $emb->write_file( $filename, $format );
 
 Output the contents of the object's pattern to the specified
 file, using the specified file format. If the filename
 is omitted, the default filename will be the last
-file that was successfully read using I<read_file()>. If
-the format is omitted, the default is 'tajima'. Returns
-0 on failure, 1 on success.
+file that was successfully read using I<read_file()>. 
+See FILE FORMATS for supported formats. Default is Tajima DST.
+Returns 0 on failure, 1 on success.
 
 =cut
 sub write_file {
@@ -302,30 +436,61 @@ sub write_file {
 		return 0;
 	}
 
-	unless(defined($file)) { $file = $self->{'filename'}; }
+	unless(defined($file)) {
+		if(defined($self->{'filename'})) {
+			$file = $self->{'filename'}; 
+		} else {
+			carp("No filename supplied");
+			return 0;
+		}
+	}
 	my $fh = IO::File->new($file, "w") or carp("Unable to write to $file") and return 0;
 
 	# for windows
 	binmode($fh);
 
-	unless(defined($type)) { $type = 'tajima'; }
+	if(defined($type)) {
+		$type = lc($type);
+	} else {
+		$type = 'tajima';
+	}
 
-	if($type eq 'tajima') {
+	if($type eq 'tajima' or $type eq 'dst') {
 		return _write_tajima_file($self, $fh);
+	} elsif($type eq 'melco' or $type eq 'exp') {
+		return _write_melco_file($self, $fh);
 	} else {
 		carp("Request to write unknown file type!");
 		return 0;
 	}
 }
 
+# output a Melco EXP file
+sub _write_melco_file {
+	my ($self, $fh) = @_;
+
+	foreach my $entry (@{$self->{'data'}{'pattern'}}) {
+		if($entry->[0] == $NORMAL) {
+			print $fh pack('H4', _encode_melco_delta($entry->[1], $entry->[2]));
+		} elsif($entry->[0] == $JUMP) {
+			print $fh pack('H4', '8004'); # this can be either 8002 or 8004
+			print $fh pack('H4', _encode_melco_delta($entry->[1], $entry->[2]));
+		} else { # color change
+			# i don't think the extra zero records are required, but most generators
+			# seem to put them in there.
+			print $fh pack('H8', '80010000');
+		}
+	}
+}
+
 # output a Tajima DST file
 sub _write_tajima_file {
 	my ($self, $fh) = @_;
 
 	# header
-	printf $fh "LA:%-16s\r", $self->{'data'}{'LA'};
-	printf $fh "ST:%07d\r", $self->{'data'}{'ST'};
-	printf $fh "CO:%03d\r", $self->{'data'}{'CO'};
+	printf $fh "LA:%-16s\r", $self->{'data'}{'label'};
+	printf $fh "ST:%07d\r", $self->{'data'}{'stitches'};
+	printf $fh "CO:%03d\r", $self->{'data'}{'color_changes'};
 
 	for('+X', '-X', '+Y', '-Y') { printf $fh "$_:%05d\r", $self->{'data'}{$_}; }
 
@@ -342,8 +507,8 @@ sub _write_tajima_file {
 	# data
 	foreach my $entry (@{$self->{'data'}{'pattern'}}) {
 		if($entry->[0] == $NORMAL or $entry->[0] == $JUMP) {
-			print $fh pack('B24', _get_move_record(@{$entry}));
-		} else {
+			print $fh pack('B24', _get_tajima_move_record(@{$entry}));
+		} else { # color change
 			print $fh pack('H6', '0000C3');
 		}
 	}
@@ -356,12 +521,12 @@ sub _write_tajima_file {
 	return 1;
 }
 
-sub _get_move_record {
+sub _get_tajima_move_record {
 	my ($jump,$x,$y) = @_;
 	my ($b0, $b1, $b2);
 
-	my %x = _get_components($x);
-	my %y = _get_components($y);
+	my %x = _get_tajima_components($x);
+	my %y = _get_tajima_components($y);
 
 	# byte 0
 	$b0.=($y{  1}?'1':'0');
@@ -401,7 +566,7 @@ sub _get_move_record {
 	return($b0.$b1.$b2);
 }
 
-sub _get_components {
+sub _get_tajima_components {
 	my ($n) = @_;
 	my ($s,%c);
 
@@ -432,14 +597,21 @@ sub draw_logo {
 		return 0;
 	}
 
-	unless(scalar(@colors) == $self->{'data'}{'CO'} + 1) {
-		carp($self->{'data'}{'CO'} + 1, " colors required, ", scalar(@colors), " colors supplied");
+	unless(scalar(@colors) == $self->{'data'}{'color_changes'} + 1) {
+		carp($self->{'data'}{'color_changes'} + 1, " colors required, ", scalar(@colors), " colors supplied");
 		return 0;
 	}
 
-	my ($x, $y) = ($self->{'data'}{'+X'}, $self->{'data'}{'y_size'} - $self->{'data'}{'+Y'});
+	my ($x, $y); 
+	
+	if($self->{'ignore_header_coordinates'}) {
+		($x, $y) = ( int($self->{'data'}{'x_size'}/2), int($self->{'data'}{'y_size'}/2));
+	} else {
+		($x, $y) = ($self->{'data'}{'+X'}, $self->{'data'}{'y_size'} - $self->{'data'}{'+Y'});
+	}
+
 	my ($new_x, $new_y);
-  
+
 	foreach my $stitch (@{$self->{'data'}{'pattern'}}) {
 		if($stitch->[0] == $NORMAL) {
 			$new_x = $x + $stitch->[1];
@@ -456,6 +628,49 @@ sub draw_logo {
 	return 1;
 }
 
+=item I<ignore_header_coordinates>
+
+  my $ignoring = $emb->ignore_header_coordinates;
+  $emb->ignore_header_coordinates( 1 );
+  
+Get or set whether to ignore the starting coordinates
+in the file header, and assume that the pattern begins
+in the center. Some programs that generate Tajima DST
+files put incorrect values into the header that cause
+the image to be off center. Enabling this will correct
+those images, but will display images with correct
+(but offcenter) starting points offset. This MUST be
+called before calling read_file.
+
+=cut
+sub ignore_header_coordinates {
+	my ($self, $ignore) = @_;
+	
+	if(defined($ignore)) {
+		$self->{'ignore_header_coordinates'} = $ignore;
+	}
+
+	return $self->{'ignore_header_coordinates'};
+}
+
+=item I<label>
+
+  my $label = $emb->label();
+  $emb->label( $new_label );
+
+Get or set the label that will be inserted into the file headers,
+if the output format supports it.
+
+=cut
+sub label {
+	my ($self, $label) = @_;
+
+	if(defined($label)) {
+		$self->{'label'} = $label;
+	}
+	return $self->{'label'};
+}
+
 =item I<size>
 
   my ($x, $y) = $emb->size();
@@ -477,7 +692,7 @@ Return the number of colors changes in the pattern.
 =cut
 sub get_color_changes {
 	my ($self) = @_;
-	return $self->{'data'}{'CO'};
+	return $self->{'data'}{'color_changes'};
 }
 
 =item I<get_color_count>
@@ -489,7 +704,7 @@ Returns the number of colors in the pattern.
 =cut
 sub get_color_count {
 	my ($self) = @_;
-	return ($self->{'data'}{'CO'} + 1);
+	return ($self->{'data'}{'color_changes'} + 1);
 }
 
 =item I<get_stitch_count>
@@ -501,7 +716,7 @@ Return the total number of stitches in the pattern.
 =cut
 sub get_stitch_count {
 	my ($self) = @_;
-	return $self->{'data'}{'ST'};
+	return $self->{'data'}{'stitches'};
 }
 
 =item I<get_end_point>
@@ -539,10 +754,22 @@ __END__
 
 =head1 FILE FORMATS
 
-Currently, only the Tajima DST file format is supported. It
-can be specifed to the I<read_file()> and I<write_file()>
-routines using the string 'tajima', if you feel like being
-specific.
+Supported file formats are Tajima DST and Melco EXP.
+These can be specifed to the I<read_file()> and I<write_file()>
+routines using the strings:
+
+Tajima DST: 'tajima' or 'dst'
+Melco EXP:  'melco' or 'exp'
+
+Strings are case-insensitive. Tajima is always the default.
+
+A note on Tajima DST files:
+It seems that many applications that produce DST files
+put incorrect information in the header. I've attempted
+to work around invalid files as much as possible, but
+occasionally there are still problems. If you have a file
+that loads in another viewer but not in this module, let me
+know and I'll see if I can fix the problem.
 
 =head1 AUTHOR
 
